#define FREQUENCY   145150          //  , 
#define SYNTH_STEP  50              //   ,  (25/50/100 KHz)

#if (100 == SYNTH_STEP)
   #define SR2R1R0     0x840000 //1000 0100 -   100      BO1  BO2 (  +8v)
#elif (50 == SYNTH_STEP)
   #define SR2R1R0     0xC40000 //1100 0100 -   50      BO1  BO2 (  +8v)
#elif (25 == SYNTH_STEP)
   #define SR2R1R0     0xA40000 //1010 0100 -   25      BO1  BO2 (  +8v)
#endif

#define FT          950.0           //     . ".0"   - !

#include "common_defs.h"
#include "Morse.h"
#include "SineTable.h"

/**************************     ****************************/
UINT16 volatile InterruptsCounter = 0;
UINT16 volatile CurrentDelay;
UINT16 volatile CurrentSineValue = 0;
UINT16 volatile Amplitude;

/*  PROGMEM  ,    
 *     (Flash) */
const UINT8 PROGMEM text[] = "CQ CQ CQ DE UA0LTB QTH PN53XC                                                    \0";

/***************************************************************************
 *  - 1   OCR1A & SWITCH ON non-inverting PWM *
 ***************************************************************************/
void SndStart(void)
{
   TCCR1 |= _BV(COM1A1);
}

/****************************************************************************
 *  - 1   OCR1A & SWITCH OFF non-inverting PWM *
 ****************************************************************************/
void SndStop(void)
{
   TCCR1     &= ~_BV(COM1A1);
   SND_PORT  &= ~_BV(PIN_OC1); // PB1 = 0 (       )
}

/**************************************************************************
 *                                                *
 **************************************************************************/
void Wait(UINT16 delay)
{
   CurrentDelay = delay;
   Amplitude = 0;                    /*     0-  */

   for (;;)
   {
      sleep_mode();                  /* ,     */
      if (InterruptsCounter > delay)
      {
         InterruptsCounter = 0;
         break;
      }
   }
}

/*************************************************************************
 *   /                              *
 *************************************************************************/
void IOinit(void)
{
   cli();    //  .       !!!
             //  -   ! */

   ACSR          = _BV(ACD);             /*      */
   TIMSK         = 0x00;                 /*     */
   InitSynthPins();
   SND_PORT_DIR |= _BV(PIN_OC1);         /*  OC1    */
   SND_PORT     &= ~_BV(PIN_OC1);        /*  OC1    0 */
}

/*************************************************************************
 * +----+----+----+-------------------------------------------------+
 * CS12CS11CS10                                         
 * +----+----+----+-------------------------------------------------+
 * 0   0   0   / 1                       
 * +----+----+----+-------------------------------------------------+
 * 0   0   1   CK                                               
 * +----+----+----+-------------------------------------------------+
 * 0   1   0   CK/8                                             
 * +----+----+----+-------------------------------------------------+
 * 0   1   1   CK/64                                            
 * +----+----+----+-------------------------------------------------+
 * 1   0   0   CK/256                                           
 * +----+----+----+-------------------------------------------------+
 * 1   0   1   CK/1024                                          
 * +----+----+----+-------------------------------------------------+
 *************************************************************************/
void PWMInit(void)
{
   TCCR1  = _BV(CS10);            /* PCK/2 */
   TCCR1 |= _BV(PWM1A);           /* PWM */
   TIMSK  = _BV(TOIE1);           /* Int T1 Overflow enabled */
}

/*************************************************************************
 *                                M A I N                                *
 *************************************************************************/
int main(void)
{
   UINT32        ctrl_word; // 24-    / LM7001
   UINT16        letter;
   UINT8         symbol_code;
   UINT8         symbol_length;
   const UINT8  *p_text;

   IOinit();

   Wait_ms(500);

   //   FM-  LM7001 14-,  . -  = 16384 (0x3FFF)
   ctrl_word = SR2R1R0 ^ (FREQUENCY/SYNTH_STEP);
   send_word(ctrl_word);

   Wait_ms(1000);

   SndStop();
   PWMInit();
   
   sei();

   Wait(MORSE_DOT_LENGTH); //     

   for (;;)
   {
      p_text = &text[0];
      for(;;)
      {
         letter = (UINT16)__LPM(p_text++);
         if (!letter) break;

         if (SPACE != letter)
         {
            letter        = (letter - EXCLAMATION_MARK)<<1;
            symbol_code   = __LPM((UINT16)(&morse_lookup_table[0]+letter++));   //p_morse_lookup_table[letter].symbol_code;
            symbol_length = __LPM((UINT16)(&morse_lookup_table[0]+letter));     //p_morse_lookup_table[letter].symbol_length;

            //   
            for (; symbol_length > 0; --symbol_length)
            {
               SndStart();
               if (symbol_code & 0x80)     //      
                  CurrentDelay = MORSE_DOT_LENGTH;
               else
                  CurrentDelay = MORSE_DASH_LENGTH;

               symbol_code <<= 1;

               Wait(CurrentDelay);         //     
               SndStop();

               Wait(MORSE_PAUSE_WITHIN_SIGN); //    
            }
            Wait(MORSE_PAUSE_BETWEEN_SIGNES); //   
         }
         else
         {
            Wait(MORSE_PAUSE_BETWEEN_WORDS);  //     
         }
      }
   }
   return 0;
}

/***************************************************************************
 *     - 1                *
 *  PWM-,    Timer/Counter1    OCR1A *
 * ,    (SREG I-bit = 1 and TOIE1 = 1)     *
 *    TIMER1_OVF_vect.                             *
 ***************************************************************************/
ISR(TIMER1_OVF_vect)
{
   if (InterruptsCounter++ < 255)
   {
      if (Amplitude < 255) Amplitude++;            //      
   }
   else if (InterruptsCounter > (CurrentDelay - 275))
   {
      if (Amplitude > 1) Amplitude--;              //      
   }

   CurrentSineValue += MORSE_FREQUENCY_STEP;       //     

   OCR1A = ((__LPM(&SmoothingTable[Amplitude]) * __LPM(&SineTable[(UINT8)(((CurrentSineValue+8) >> 4)&(0x00FF))]))+256) >> 8; //   
}


/**************************************************************************
 *   24-      LM7001  *
 **************************************************************************/
void send_word(UINT32 word)
{
   UINT32 mask = 0x00000001; //  LSB 
   UINT8  i;

   SetCEHigh();      //     / LM7001

   for (i = 0; i < 24; i++)
   {
      if (mask & word)
      {
         SetDataHigh();
      }
      else
      {
         SetDataLow();
      }
      mask <<= 1;

      SetCLKHigh();  // 
      SetCLKLow();   // *
   }

   SetCELow();       //     / LM7001
   SetDataLow();
}

/*****************************
 *        *
 *****************************/
void Wait_us(UINT16 pause)
{
   UINT16 i;
   for (i = 0; i < pause; i++)
   {
      __asm__ volatile("nop");
      __asm__ volatile("nop");
      __asm__ volatile("nop");
      __asm__ volatile("nop");
      __asm__ volatile("nop");
      __asm__ volatile("nop");
      __asm__ volatile("nop");
      __asm__ volatile("nop");
   }
}

/*****************************
 *        *
 *****************************/
void Wait_ms(UINT16 pause)
{
   UINT16 i;
   for (i = 0; i < pause; i++)
   {
      Wait_us(1000);
   }
}
